#!/bin/sh

# here is readahead in KB that we use
# this seems huge but for a modern RAID volume that can do 1 GB/s,
# this is just 65 msec of data, enough to optimize away some seeks on
# a multi-stream workload but not enough to kill response time

tuned_ra=4096
default_ra=128
# can't set nr_requests for device-mapper device only for underlying block device
#tuned_nrq=512
#default_nrq=128
limits_conf="/etc/security/limits.conf"

. /etc/tune-profiles/functions

# set readahead in first CLI parameters on ALL Gluster brick devices

OK=0

set_brick_ra() {
  ra=$1
  bricklist='/tmp/local-bricks.list'
  if [ ! -x $gluster_cfg_dir ] ; then return ; fi
  rm -f $bricklist
  touch $bricklist
  for d in `find /var/lib/glusterd/vols -name bricks -type d 2>/tmp/e ` ; do
       (cd $d ; ls | grep `hostname -s` | awk -F: '{ print $2 }' | sed 's/\-/\//g' \
          >> $bricklist 2>> /tmp/bricks.err)
  done
  echo -n "setting readahead to $ra on brick devices: "
  for mountpt in `cat $bricklist`  ; do
    next_dev=`grep $mountpt /proc/mounts | awk '{ print $1 }'`
    if [ -L $next_dev ] ; then 
      # device name is really a symbolic link to a device-mapper device, use the link target
      device_path=`readlink $next_dev`
    else
      device_path="$next_dev"
    fi
    device_name=`basename $device_path`
    echo -n " $device_name"
    echo $ra > /sys/block/$device_name/queue/read_ahead_kb
  done
  echo
}

start() {
	set_cpu_governor performance

	# Find non-root and non-boot partitions, disable barriers on them
	rootvol=$(df -h / | grep "^/dev" | awk '{print $1}')
	bootvol=$(df -h /boot | grep "^/dev" | awk '{print $1}')
	volumes=$(df -hl --exclude=tmpfs | grep "^/dev" | awk '{print $1}')

	nobarriervols=$(echo "$volumes" | grep -v $rootvol | grep -v $bootvol)
	remount_partitions nobarrier $nobarriervols
        set_brick_ra $tuned_ra
        ( grep -v 'soft nofile' $limits_conf | grep -v 'hard nofile' ; \
          echo '* soft nofile 4096' ; \
          echo '* hard nofile 16384' ) > /tmp/limits.conf
        mv $limits_conf $limits_conf.sav
        mv /tmp/limits.conf $limits_conf
	return 0
}

stop() {

	# Find non-root and non-boot partitions, re-enable barriers
	rootvol=$(df -h / | grep "^/dev" | awk '{print $1}')
	bootvol=$(df -h /boot | grep "^/dev" | awk '{print $1}')
	volumes=$(df -hl --exclude=tmpfs | grep "^/dev" | awk '{print $1}')

	nobarriervols=$(echo "$volumes" | grep -v $rootvol | grep -v $bootvol)
	remount_partitions barrier $nobarriervols
        set_brick_ra $default_ra 
	restore_cpu_governor
        grep -v 'soft nofile' $limits_conf | grep -v 'hard nofile' > /tmp/limits.conf
        mv $limits_conf $limits_conf.sav
        mv /tmp/limits.conf $limits_conf
	return 0
}

process $@
